@@ -2,7 +2,7 @@
 #
 #   grosetta.py -- Front-end script for submitting ROSETTA jobs to SMSCG.
 #
-#   Copyright (C) 2010, 2011 GC3, University of Zurich
+#   Copyright (C) 2010, 2011, 2012 GC3, University of Zurich
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -23,9 +23,11 @@ It uses the generic `gc3libs.cmdline.SessionBasedScript` framework.
 
 See the output of ``grosetta --help`` for program usage instructions.
 """
-__version__ = '1.0 (SVN $Revision$)'
+__version__ = '2.0.0 version (SVN $Revision$)'
 # summary of user-visible changes
 __changelog__ = """
+  2012-02-28:
+    * Use option ``-R`` to request a specific release of Rosetta.
   2011-03-28:
     * No longer require any input file on the command line: e.g., ``grosetta -l``
       will just list the status of jobs in the current session.  On the other hand,
@@ -36,16 +38,23 @@ __changelog__ = """
 __author__ = 'Riccardo Murri <riccardo.murri@uzh.ch>'
 __docformat__ = 'reStructuredText'
 
+
+
+# workaround Issue 95, see: http://code.google.com/p/gc3pie/issues/detail?id=95
+if __name__ == '__main__':
+    import grosetta
+    grosetta.GRosettaScript().run()
+
+
+# stdlib imports
 import os
 import os.path
 import sys
 
-
-## interface to Gc3libs
-
+# interface to Gc3libs
 import gc3libs
 from gc3libs.application.rosetta import RosettaApplication
-from gc3libs.cmdline import SessionBasedScript
+from gc3libs.cmdline import SessionBasedScript, positive_int
 
 
 
@@ -62,7 +71,7 @@ The `grosetta` command keeps a record of jobs (submitted, executed and
 pending) in a session file (set name with the '-s' option); at each
 invocation of the command, the status of all recorded jobs is updated,
 output from finished jobs is collected, and a summary table of all
-known jobs is printed.  New jobs are added to the session if the number 
+known jobs is printed.  New jobs are added to the session if the number
 of wanted decoys (option '-P') is raised.
 
 Options can specify a maximum number of jobs that should be in
@@ -73,39 +82,41 @@ Note: the list of INPUT and OUTPUT files must be separated by ':'
 (on the shell command line, put a space before and after).
     """
 
-    usage = "%(prog)s [options] FLAGSFILE INPUT ... [: OUTPUT ...]"
-
     def __init__(self):
         SessionBasedScript.__init__(
             self,
+            usage = "%(prog)s [options] FLAGSFILE INPUT ... [: OUTPUT ...]",
             version = __version__, # module version == script version
             application = RosettaApplication,
             )
 
     def setup_options(self):
-        self.add_param("-P", "--total_decoys", type=int, dest="total_decoys", 
-                       default=1,
+        self.add_param("-P", "--total-decoys", dest="total_decoys",
+                       type=positive_int, default=1,
                        metavar="NUM",
                        help="Compute NUM decoys per input file (default: %(default)s)."
                        )
-        self.add_param("-p", "--decoys-per-job", type=int, dest="decoys_per_job", 
-                       default=1,
+        self.add_param("-p", "--decoys-per-job", dest="decoys_per_job",
+                       type=positive_int, default=1,
                        metavar="NUM",
                        help="Compute NUM decoys in a single job (default: %(default)s)."
                        " This parameter should be tuned so that the running time"
                        " of a single job does not exceed the maximum wall-clock time."
                        )
-        self.add_param("-x", "--protocol", dest="protocol", default="minirosetta.static",
+        self.add_param("-x", "--protocol", dest="protocol",
+                       type=str, default="minirosetta.static",
                        metavar="PROTOCOL",
                        help="Run the specified Rosetta protocol/application; default: %(default)s")
+        self.add_param("-R", "--release",
+                       type=str, dest="rosetta_release", default="3.1",
+                       metavar="NAME",
+                       help="Numerical suffix to identify which version of Rosetta should be requested."
+                       " For example: '-e 20110622' will run rosetta-svn20110622."
+                       " (default: %(default)s)"
+                       )
 
     def parse_args(self):
-        if self.params.total_decoys < 1:
-            raise RuntimeError("Argument to option -P/--total-decoys must be a positive integer.")
         self.instances_per_file = self.params.total_decoys
-
-        if self.params.decoys_per_job < 1:
-            raise RuntimeError("Argument to option -p/--decoys-per-job must be a positive integer.")
         self.instances_per_job = self.params.decoys_per_job
         self.extra['number_of_decoys_to_create'] = self.params.decoys_per_job
 
@@ -135,7 +146,7 @@ Note: the list of INPUT and OUTPUT files must be separated by ':'
 
             # make flags file path absolute
             if not os.path.isabs(self.flags_file):
-                self.flags_file = os.path.join(os.getcwd(), self.flags_file)
+                self.flags_file = os.path.abspath(self.flags_file)
             if not os.path.exists(self.flags_file):
                 raise RuntimeError("Flags file '%s' does not exist." % self.flags_file)
             self.log.info("Using flags file '%s'", self.flags_file)
@@ -149,15 +160,15 @@ Note: the list of INPUT and OUTPUT files must be separated by ':'
             else:
                 # make paths absolute
                 if not os.path.isabs(path):
-                    path = os.path.join(os.getcwd(), path)
+                    path = os.path.abspath(path)
                 inputs_.append(path)
         self.inputs = inputs_
-        self.log.debug("Gathered input files: '%s'" % str.join("', '", inputs))
+        #self.log.debug("Gathered input files: '%s'" % str.join("', '", inputs))
 
-    def process_args(self, extra):
+    def new_tasks(self, extra):
         ## compute number of decoys already being computed in this session
         decoys = 0
-        for task in self.tasks:
+        for task in self.session:
             start, end = task.jobname.split('--')
             decoys += int(end) - int(start)
         self.log.debug("Total no. of decoys already scheduled for computation: %d", decoys)
@@ -170,21 +181,16 @@ Note: the list of INPUT and OUTPUT files must be separated by ':'
                               decoys, self.params.total_decoys - decoys)
             # create new jobs and add them to session
             for nr in range(decoys, self.params.total_decoys, self.params.decoys_per_job):
-                jobname = ("%d--%d" 
-                           % (nr, min(self.params.total_decoys, 
+                jobname = ("%d--%d"
+                           % (nr, min(self.params.total_decoys,
                                       nr + self.params.decoys_per_job - 1)))
                 # yield new job to construct to `self._main()`
                 yield (
-                    jobname, RosettaApplication, 
+                    jobname, RosettaApplication,
                     # args
-                    (self.params.protocol, self.inputs, self.outputs),
+                    (self.params.protocol, self.params.rosetta_release, self.inputs, self.outputs),
                     # kwargs
                     {'arguments':[ '-out:nstruct', str(self.params.decoys_per_job) ],
                      'flags_file':self.flags_file,
                      },
                     )
-
-
-## run it
-if __name__ == '__main__':
-    GRosettaScript().run()
