@@ -3,7 +3,7 @@
 """
 Specialized support for computational jobs running simple demo.
 """
-# Copyright (C) 2009-2011 GC3, University of Zurich. All rights reserved.
+# Copyright (C) 2009-2012 GC3, University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -20,15 +20,17 @@ Specialized support for computational jobs running simple demo.
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #
 __docformat__ = 'reStructuredText'
-__version__ = '1.0 (SVN $Revision: 1508 $)'
+__version__ = '2.0.0 version (SVN $Revision: 2991 $)'
 
 
+import os
+import os.path
+from pkg_resources import Requirement, resource_filename
+
 import gc3libs
 import gc3libs.application
 from gc3libs.exceptions import *
-import os
-import os.path
-from pkg_resources import Requirement, resource_filename
+from gc3libs.quantity import GB, hours
 
 
 class Square(gc3libs.Application):
@@ -36,6 +38,9 @@ class Square(gc3libs.Application):
     Square class, takes a filename containing a list of integer to be squared.
     writes an output containing the square of each of them
     """
+
+    application_name = 'demo'
+
     def __init__(self, x):
         # src_square_sh = resource_filename(Requirement.parse("gc3utils"),
         #                                   "gc3libs/etc/square.sh")
@@ -46,30 +51,24 @@ class Square(gc3libs.Application):
 
         _outputs = []
 
-        # kw.setdefault('stdout', 'stdout.txt')
-        # kw.setdefault('stderr', 'stderr.txt')
-        
-        #  gc3libs.Application.__init__(self, executable, arguments, inputs, outputs, output_dir, **kw):
-
+        # extra_args.setdefault('stdout', 'stdout.txt')
+        # extra_args.setdefault('stderr', 'stderr.txt')
 
         gc3libs.Application.__init__(self,
-                                     executable = "/usr/bin/expr",
-                                     arguments = [str(x), "*", str(x)],
+                                     arguments = ["/usr/bin/expr", str(x), "*", str(x)],
                                      inputs = [],
                                      outputs = [],
                                      output_dir = None,
                                      stdout = "stdout.txt",
                                      stderr = "stderr.txt",
                                      # set computational requirements. XXX this is mandatory, thus probably should become part of the Application's signature
-                                     requested_memory = 1,
+                                     requested_memory = 1*GB,
                                      requested_cores = 1,
-                                     requested_walltime = 1
+                                     requested_walltime = 1*hours,
                                      )
 
 
 
-gc3libs.application.register(Square, 'square')
-
 ## main: run tests
 
 if "__main__" == __name__:
